# Cpanel to Eenos Migration Tool ( cptoeenos a.k.a cp2eenos)

This tool will help you to convert cpanel hosting accounts into Eenos Hosting account backup format so that you can migrate cpanel accounts to any Eenos control panel server.

[Eenos](https://eenos.com/) is a modern high-performance hosting control panel and a total replacement of your old traditional hosting control panels.

### cptoeens will do the following :

- Convert Single Cpanel account into Eenos backup.
- Convert Cpanel backup file into Eenos backup.
- Convert WHM Hosting packages into Eenos Hosting packages.
- Convert the Full Cpanel server accounts and packages into Eenos Format.

## Installing cptoeenos

**cptoeenos** can be installed on any Linux/Unix operating system with  Python3 installed (3.6 or higher). 

To install cptoeenos run the following command:

```
pip install cptoeenos

```
To convert a live cpanel account,  WHM package or the whole cpanel server, please install this tool in your cpanel server.

To convert a cpanel backup file, you can install it in your Eenos server or any Linux server.

## What is included with the conversion?

**cptoeenos / cp2eenos** will convert the whole data on your cpanel account into Eenos account backup format. This migration process includes the following:

- Convert cpanel domains, subdomains, parked domains and addon domains.
- Convert cpanel MySQL database and users.
- Convert cpanel PostgreSQL database and users.
- Convert cpanel Email accounts, forwarders and email pips
- Convert cpanel Email filters and domain filters
- Convert cpanel FTP accounts
- Convert cpanel SSL certificates
- Convert Reseller settings and limits
- Convert WHM hosting packages
- Convert Reseller hosting packages.
- Convert Cron jobs
- Convert DNS
- Convert DKIM keys
- Copy SPF settings
- Option to skip home directory
- Option to skip MySQL and PostgreSQL databases
- Option to skip Email accounts
- Suspended accounts will be kept as suspended itself

### What is not included?

The cpanel mailman2 mailing lists were not supported. The data like logs, bandwidth usage and analytics stats are excluded, because those are not required to migrate. 

This will migrate your cpanel account to Eenos and keep it as is. There is no need to configure anything manually in your Eenos server. The resellers are kept with same reseller privileges and reseller packages.

## cptoeenos / cp2eenos command options :
The following command options are available with this tool.

```
Command(s):
  -f, --full            Convert all Cpanel accounts and hosting packages
  -u user, --user user  Convert a single Cpanel user/reseller account 
  -b backup_file, --backupfile backup_file
                        Convert a Cpanel backup file( .tar , .tar.gz)
  -p, --packages        Convert all WHM hosting packages

Backup convert option(s):
  --skipmysql           Exclude MySQL database(s)
  --skippgsql           Exclude PgSQL database(s)
  --skiphomedir         Exclude Homedir
  --skipemail           Exclude Email accounts 
  --skipresellerconfig  Exclude reseller privileges

Upload Option(s):
  -s path_to_store, --saveto path_to_store
                        A folder location to save the converted backups 

Option(s):
  -h, --help            show this help message and exit
  -d, --demon           Run convert process as daemon service
  -w workdir, --workdir workdir
                        A folder to generate temporary backup files,
                         Working directory: Default /root
  -pu path_to_script, --postupload path_to_script
                        A script path name to run after conversion.
                        You can use a bash script to upload the
                        converted backups to remote server.

```


## How to convert a Cpanel backup file to Eenos Backup?

You can convert a cpanel backup file to Eenos backup file. The cpanel backup file must be a **.tar** or **.tar.gz** file.

### Syntax :

```
cptoeenos --saveto /path-to-save-converted-backup --backupfile /path-to-cpanel-account-backup-file

```

### Example :

We are going to convert a cpanel backup name **backup-10.14.2023_16-20-05_mycpuser.tar.gz** and store the converted backup in the folder **/backup**. The command is as follows:

```
cptoeenos --saveto /backup --backupfile /home/backup-10.14.2023_16-20-05_mycpuser.tar.gz

```
This will generate a backup on **/backup/eenosbkp-myuser.tar.gz**. You may upload this backup file to your  Eenos server and restore it.

## How to convert WHM hosting packages?

You can convert all your cpanel/WHM hosting packages into Eenos hosting packages with the following command :

```
cptoeenos --saveto /backup --packages

```

The above command will convert the WHM packages and upload them to the  folder **/backup/EenosHostingPackages**

You may now copy all the .yaml files from this folder into **/var/eenos/packages/** of your Eenos server and  change the ownership of those files into **eenos**

> [!NOTE]
> If the  WHM package name contains white spaces or dots, it will be replaced with a hyphen in Eenos during the package conversion.

## How to convert a Cpanel account from Cpanel server?

You can convert a live cpanel account into Eenos backup file. You may run the following command to backup a live cpanel account

```
cp2eenos --saveto /backup --user cpusername

```

where, **cpusername** is the cpanel account user name.

The above command will generate an Eenos compatible backup of the user cpusername and place it into the location **/backup/eenosbkp-cpusername.tar.gz**

## How to convert the whole cpanel server into Eenos?

You can convert your whole  cpanel accounts and hosting packages into Eenos backup format using the following command,

```
cp2eenos --saveto /backup  --full

```

## How to run the conversion in server background?

You may use the flag **--demon** or **-d** along with the conversion command to perform the task in the server background.

### Example : 

To run the whole conversion package of the complete cpanel server in the background

```
cp2eenos --saveto /backup  --full --demon

```

## Automatic upload of converted backups into remote server

You can attach a post-upload script for the conversion process so that your script will be executed after the backup conversion.

To run a post upload script , you may use the  flag **--postupload path_to_script** or **-pu path_to_script**

### Example :

Convert the whole cpanel accounts and WHM packages then upload to a remote server using my post upload script **/root/postupload.sh**

```
cp2eenos --saveto /backup  --full --demon --postupload /root/postupload.sh

```

Here the conversion process will run in the server background and execute the post-upload script soon after finishing the cpanel account conversion process. 
The post-upload script will be provided with a command line argument which is the save path  ( **--saveto**) of the backups.

## How to restore the converted backup in Eenos server?

You may upload the converted backup to the  Eenos server and  use the script **/scripts/restoreacct** to restore the backup

### Example:

To restore the backup file uploaded to /home/eenosbkp-cpuser.tar.gz  , run the following command from Eenos server.

```
/scripts/restoreacct /home/eenosbkp-cpuser.tar.gz 

```

# Read More or Get help

* [Documentation](https://docs.eenos.com/books/migration-tools/chapter/cpanel-to-eenos-migration-tool-cptoeenos-aka-cp2eenos)
* [Bug Report](https://eenos.com/contact/)
* [License - GPL V3](https://www.gnu.org/licenses/gpl-3.0.en.html)
* [Eenos Hosting Control Panel](https://eenos.com/)
