import os
import shutil
from pprint import pprint
import glob
import re
from cptoeenos import tools,conf,commands,cpanel,tarops
import hashlib
import tarfile

def get_tempdir(fname):
    return str(hashlib.sha1(str.encode(fname)).hexdigest()[0:15])

def extract_tar(tfile,to,log):
    result=None
    if tarfile.is_tarfile(tfile):
        try:
            if not os.path.isdir(to):
                os.makedirs(to)
            if str(tfile).endswith('.tar'):
                cmd = 'tar --strip-components=1 -xf '+tfile+' -C '+to
            else:
                cmd = 'tar --strip-components=1 -xzf '+tfile+' -C '+to
            log("Extracting backup file ..... please wait...")
            log(cmd)
            commands.live(cmd,log)
            result=to
        except Exception as e:
            log(str(e))
            result=None
            pass
    else:
        log("Backup is not a tar file . unable  to extract  !!!")
    return result

def find_cpuser(backup,log):
    cpuser=None
    if os.path.isdir(backup+'/cp/'):
        ulist=glob.glob(backup+'/cp/*')
        if bool(ulist):
            try:
                cpuser=ulist[0].split('/')[-1]
            except Exception as e:
                log(str(e))
                cpuser=None
                pass
    return cpuser


def convert(choice,log):    
    backupfile=choice['backupfile']
    log("------------------------")
    log("Backup File conversion : %s" %backupfile)
    log("------------------------")
    basedir=choice['myworkdir']
    wname=get_tempdir(backupfile)    
    workdir=basedir+'/'+wname    
    result=None
    try:
        if os.path.isdir(workdir):
            log("Cleaning work directory : %s" %workdir)
            shutil.rmtree(workdir)
    except Exception as e:
        log(str(e))
        pass
    cresult=extract_tar(backupfile,workdir,log)
    if cresult and os.path.isdir(cresult):
        cpuser=find_cpuser(cresult,log)
        if cpuser:
            log("Cpanel user name found : %s" %cpuser)
            backup=cpanel.start(cpuser,cresult,choice,log,live=False)                                    
            if backup:
                outputfile=backup+'.tar.gz'
                archfile=tarops.make_gzip_tar_from_dir(outputfile,backup,log)
                if archfile:
                    pkgname=str(archfile).split('/')[-1]
                    log("Cpanel account converted successfully  : %s " %pkgname)
                    uploadpath=str(choice['saveto'])+'/'+pkgname
                    try:
                        log("Uploading Eenos backup to : %s" %uploadpath)
                        shutil.copy(archfile,uploadpath)
                        result=uploadpath
                    except Exception as e:
                        log(str(e))
                        result=None
                        pass
                    try:
                        log("Cleaning temporary backup files .. %s" %backup)                    
                        shutil.rmtree(backup)
                        os.remove(archfile)
                    except Exception as e:
                        log(str(e))
                        pass
                    if result:
                        log("===================================")
                        log("Your Eenos Backup is : %s" %result)
                        tools.get_hash(result,log)
                    else:
                        log("Cpanel to Eenos conversion failed.")                    
                else:
                    log("Unable to create archive backup file , backup failed")
                    result=None
            else:
                log("Cpanel to Eenos account conversion failed !!!") 
        else:
            log("Unable to get cpanel user name from backup !!!")
    else:
        log("Unable to extract the backup file.")
    # print(workdir)    
    # pprint(choice)
    