import os,sys
from cptoeenos import elog,init_convert
from datetime import datetime
import argparse
import pathlib
from pprint import pprint
import time
import atexit
import signal

# EenosConverter(cmd,pidfile,logfile)
class EBackupMonitor:
    def __init__(self,cmd,pidfile,logfile): 
        self.pidfile = pidfile
        self.cmd = cmd
        self.logfile = logfile
        logger,logpath=elog.get_daemon_logger(logfile)
        self.log=logger.info 
        self.logpath=logpath    

    def daemonize(self,logpath):
        try: 
            pid = os.fork() 
            if pid > 0:
                sys.exit(0) 
        except OSError as err: 
            sys.stderr.write('CptoEenos Conversion Demon Fork  failed: {0}\n'.format(err))
            sys.exit(1)
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0) 
        except OSError as err:
            sys.stderr.write('fork #2 failed: {0}\n'.format(err))
            sys.exit(1) 
        sys.stdout.flush()
        sys.stderr.flush()
        si = open(os.devnull, 'r')
        #so = open(os.devnull, 'a+')
        flogfile=str(self.logpath)        
        so = open(flogfile, 'a+')
        se = open(os.devnull, 'a+')

        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        # write pidfile
        atexit.register(self.delpid)

        pid = str(os.getpid())
        with open(self.pidfile,'w+') as f:
            f.write(pid + '\n')
    
    def delpid(self):
        os.remove(self.pidfile)

    def start(self):                      
        try:           
            with open(self.pidfile,'r') as pf:
                pid = int(pf.read().strip())
        except IOError:
            pid = None
        if pid:
            message = "CptoEenos Conversion Daemon already running  with pidfile {0}  == \n"
            sys.stderr.write(message.format(self.pidfile))
            sys.exit(1)        
        # Start the daemon
        print("Starting Cpanel to Eenos  Conversion Daemon with pid %s .. done" %(self.pidfile))
        print("Log file : %s" %self.logpath)
        # self.log("Starting CptoEenos  Convertsion Daemon with pid %s .. done" %(self.pidfile))                  
        self.daemonize(self.logpath)
        self.run(self.cmd,self.log,self.logpath)

    def stop(self):
        # Get the pid from the pidfile
        try:
            with open(self.pidfile,'r') as pf:
                pid = int(pf.read().strip())
        except IOError:
            pid = None
    
        if not pid:
            message = "Cpanel to Eenos Conversion daemon ( {1} )  is not running \nPidfile {0} does not exist.\n"
            sys.stderr.write(message.format(self.pidfile,self.args))
            return # not an error in a restart
        # Try killing the  Monitor daemon process    
        try:
            self.log("Stopping Cpanel to Eenos Conversion daemon  .. done")
            # self.log("Stopping : %s " %(self.cmd)) 
            # Get the pid from the pidfile
            while 1:
                os.kill(pid, signal.SIGTERM)
                time.sleep(0.1)
        except OSError as err:
            e = str(err.args)
            if e.find("No such process") > 0:
                if os.path.exists(self.pidfile):
                    os.remove(self.pidfile)
            else:
                self.log(str(err.args))
                sys.exit(1)
    def run(self,cmd,log,logpath):
        """
        Eenos bg runner override to Main Class
        """

class EenosConverter(EBackupMonitor):
    def run(self,cmd,log,logfile): 
        init_convert.start(cmd,log,logfile)           
        sys.exit(0)

def validate_cpuser(arg_value):
    if not os.path.isfile('/var/cpanel/users/'+arg_value):
        raise argparse.ArgumentTypeError("Cpanel user not found")
    else:
        return arg_value
def validate_file(value):
    if os.path.isfile(value):
        if str(value).endswith('.tar') or str(value).endswith('.tar.gz'):
            return value
        else:
            raise argparse.ArgumentTypeError("Invalid Cpanel backup file format")
    else:
        raise argparse.ArgumentTypeError("Invalid Cpanel backup")
    
def options(default=False):
    parser = argparse.ArgumentParser(description='Cpanel to Eenos Conversion Tool: \n To convert cpanel accounts and backup files to Eenos backup file.',usage='%(prog)s [options] [values]',formatter_class=argparse.RawTextHelpFormatter)
    optional=parser._action_groups.pop()  
    parser._optionals.title='Option(s)'    
    main_cmd = parser.add_argument_group("Command(s)")  
    main=main_cmd.add_mutually_exclusive_group(required=True)
    main.add_argument('-f','--full', action='store_true',  help='Convert all Cpanel accounts and hosting packages')   
    main.add_argument('-u','--user',metavar="user",type=validate_cpuser,help='Convert a single Cpanel user/reseller account ')
    main.add_argument('-b','--backupfile',metavar="backup_file",type=validate_file,help='Convert a Cpanel backup file( .tar , .tar.gz)')
    main.add_argument('-p','--packages',action='store_true',  help='Convert all WHM hosting packages') 

    full_options=parser.add_argument_group("Backup convert option(s)")
    full_options.add_argument('--skipmysql', action='store_true',  help='Exclude MySQL database(s)')
    full_options.add_argument('--skippgsql', action='store_true',  help='Exclude PgSQL database(s)') 
    full_options.add_argument('--skiphomedir', action='store_true',  help='Exclude Homedir') 
    full_options.add_argument('--skipemail', action='store_true',  help='Exclude Email accounts ')
    full_options.add_argument('--skipresellerconfig', action='store_true',  help='Exclude reseller privileges')

    upload_opt=parser.add_argument_group("Upload Option(s)")
    upload=upload_opt.add_mutually_exclusive_group(required=True)
    upload.add_argument('-s','--saveto',metavar="path_to_store" ,type=pathlib.Path,  help='A folder location to save the converted backups ')  

    optional.add_argument('-d','--demon',action='store_true',help="Run convert process as daemon service")
    optional.add_argument('-w','--workdir',metavar="workdir",type=pathlib.Path,default=str(pathlib.Path.home()),help="A folder to generate temporary backup files,\n Working directory : Default %s" %str(pathlib.Path.home()))
    optional.add_argument('-pu','--postupload',metavar="path_to_script" ,type=pathlib.Path,  help='A script path name to run after conversion.\nYou can use a bash script to upload the\nconverted backups to remote server.')
    parser._action_groups.append(optional)
    args=None
    if default:        
        now = datetime.now()
        print('Copyright(c) 2023-'+str(now.year)+', eenos.com')
        parser.print_help()
        sys.exit(1)
    else:
        try:
            args = parser.parse_args()
            return vars(args)            
        except Exception as e:
            print(str(e))
            # parser.print_help(sys.stderr)
            sys.exit(0)       

def run_demon(cmd,logfile="cptoeenos.log"):    
    pidfile='/run/cptoeenos.pid'
    if os.path.isfile(pidfile):
        past=time.time() - 24*60*60        
        if os.path.getmtime(pidfile) >= past:
            print("A conversion process  is already running with pid : %s" %pidfile)                
            sys.exit(0)
        else:
            os.unlink(pidfile)    
    monitor = EenosConverter(cmd,pidfile,logfile)
    monitor.start()
    

def run_live(cmd,logfile="cptoeenos.log"):
    logger,logpath=elog.get_logger(logfile)
    log=logger.info    
    init_convert.start(cmd,log,logpath)    
    logger.handlers.clear()

def run():
    if not os.path.isfile('/usr/bin/rsync'):
        print("Command not found : /usr/bin/rsync")
        print("Please install rsync first")
        sys.exit(0)
    if not sys.argv[1:]:        
        options(default=True)        
        sys.exit(0)
    else:
        cmd=options()        
        if bool(cmd['demon']):
            run_demon(cmd)
        else:
            run_live(cmd)
        