import os
from pprint import pprint
import shutil,glob,re
from cptoeenos import tools,conf,commands,cpanel,security,pkgconvert
import dns.zone
from dns.exception import DNSException
from dns.rdatatype import RdataType
from datetime import datetime
import ipaddress
import ssl
import warnings
warnings.filterwarnings('ignore', message='.*cryptography', )
import OpenSSL
import re
import socket

def setup_work_dir(workdir,log):
    result=False    
    if not os.path.isdir(workdir):
        try:
            os.makedirs(workdir)
            result=True
        except Exception as e:
            log(str(e))
            result=False
            pass
    else:
        try:
            shutil.rmtree(workdir)
            os.makedirs(workdir)
            result=True
        except Exception as e:
            log(str(e))
            result=False
            pass        
    return result
def get_shadow(backup,workdir,log):
    cp_shadow=backup+'/shadow'
    e_shadow=workdir+'/shadow'
    if os.path.isfile(cp_shadow):
        try:
            log("Coping cpanel account password .. ")
            shutil.copy(cp_shadow,e_shadow)
            security.safe_chmod(e_shadow,0o600)
        except Exception as e:
            log(str(e))
            pass

def get_pan_name(pname,log):
    if pname=='default':
        result='Default'
    elif bool(re.search(r"[\s.]+",pname)): 
        result=tools.convert_package_feature_name(pname,log)
    else:
        result=pname
    return result
def get_ips(backup,domain,log):
    log("Collecting IP Address(s) details")
    ipdata={}
    ipdata['ipv4']='127.0.0.1'
    ipdata['ipv6']='0000:0000:0000:0000:0000:0000:0000:0001'
    ipdata['status']='off'
    ipdata['status6']='off'
    domfile=backup+'/userdata/'+domain
    domdata=tools.readyaml(domfile,log)
    if 'ip' in domdata and bool(domdata['ip']):
        ipdata['ipv4']=domdata['ip']
        log("IPv4 found on account : %s" %domdata['ip'])
        #Cpanel server check for dedicated ipv4 for live conversion
        cpipfile='/etc/domainips'
        # cpipfile='/etc/domainips.delete'
        if os.path.isfile(cpipfile):
            haveip=False
            ipcheck=str(domdata['ip'])+':'
            ipdatas=tools.readplain(cpipfile,log)
            for dips in ipdatas:               
                if str(dips).startswith(ipcheck):
                    ipdomain=dips.split(" ")[-1].strip()
                    if ipdomain == domain:
                        haveip=True
                        log("Found dedicated IPv4 on this account ")
            if haveip:
                ipdata['status']="on"
        # end of live ipv4 check
    if 'ipv6' in domdata and bool(domdata['ipv6']):        
        for iv6 in domdata['ipv6']:
            log("Ipv6  address found : %s" %iv6)
            ipdata['ipv6']=iv6
            if 'dedicated' in domdata['ipv6'][iv6] and domdata['ipv6'][iv6]['dedicated'] == 1:
                log("IPv6 address is dedicated.")
                ipdata['status6']='on'    
    return ipdata

def get_homedir(user,backup,domain,log):
    home='/home/'+user
    domfile=backup+'/userdata/'+domain
    domdata=tools.readyaml(domfile,log)
    if 'homedir' in domdata and bool(domdata['homedir']):
        home=domdata['homedir']
    return home

def get_nameserver(user,backup,domain,log):
    nss=list()
    zonefile=backup+'/dnszones/'+domain+'.db'
    if os.path.isfile(zonefile):        
        try:
            zone = dns.zone.from_file(zonefile, domain)            
            for name,node in zone.nodes.items():
                # print(name)
                rdatasets = node.rdatasets
                for rdataset in rdatasets:
                    for rdata in rdataset:
                        # print(rdata.rdtype.name)
                        # if str(rdataset.rdtype) == 'RdataType.NS':
                        if rdataset.rdtype.name == 'NS':
                            # print(rdata.target)
                            nstr=str(rdata.target)  
                            if nstr.endswith('.'):
                                nstr=nstr[:-1]                          
                            nss.append(nstr)                             
        except Exception as e:
            log(str(e))
            pass
    if not bool(nss):
        nss.append("dns1."+domain)
        nss.append("dns2."+domain)
    # pprint(nss)
    return nss

def convert_utc_todate(item,log):
    log("Collecting account setup time ")
    uts=(datetime.now() - datetime(1970, 1, 1)).total_seconds()
    time=datetime.utcfromtimestamp(uts).strftime('%Y-%m-%d %H:%M:%S')
    data=item.split("=")
    try:
        uts=int(data[-1])        
        time=datetime.utcfromtimestamp(uts).strftime('%Y-%m-%d %H:%M:%S')
    except Exception as e:
        log(str(e))
        pass
    log("Account setup time is - %s" %time)
    return time

def is_reseller(user,backup,log):
    status=False
    rfile=backup+'/resellerconfig/resellers'
    if os.path.isfile(rfile) and os.stat(rfile).st_size >0:
        rdata=tools.get_reseller(rfile,log)
        if user in rdata:
            status=True
    return status
def convert_bw_to_eenos(bw,log):
    if bw <= 1048576:
        return str(1)
    else:
        cb=int(bw/1048576)
        log("Converting bandwidth from bits to MB ( %d bits => %d MB )" %(bw,cb))
        return str(cb)
def convert_quota_to_eenos(q,log):
    if q <= 1024:
        return str(1)
    else:
        cb=int(q/1024)
        log("Converting Disk QUota from bytes to MB ( %d bytes => %d MB )" %(q,cb))
        return str(cb)
def convert_cp_userdata(user,backup,data,log):
    udata=dict()
    ufile=backup+'/cp/'+user    
    pdata=tools.readplain(ufile,log)
    isconverted=False
    if bool(pdata):
        cl=False
        if any("lve_" in s for s in pdata):
            cl=True        
        else:
            cl=False
        for item in pdata:
            if cl:
                if str(item).endswith("=DEFAULT"):
                    new_item=str(item).replace("=DEFAULT","=default")                    
                    pdata[pdata.index(item)]=new_item
                if str(item).startswith('STARTDATE='):
                    newdate=convert_utc_todate(item,log)
                    pdata[pdata.index(item)]='STARTDATE='+str(newdate)
        if 'DISK_BLOCK_LIMIT=0' in pdata:
            pdata[pdata.index('DISK_BLOCK_LIMIT=0')]='DISK_BLOCK_LIMIT=unlimited'
        if 'HASDKIM=1' in pdata:
            pdata[pdata.index('HASDKIM=1')]='HASDKIM=on'
        if 'HASDKIM=0' in pdata:
            pdata[pdata.index('HASDKIM=0')]='HASDKIM=off'
        if 'HASSPF=0' in pdata:
            pdata[pdata.index('HASSPF=0')]='HASSPF=off'
        if 'HASSPF=1' in pdata:
            pdata[pdata.index('HASSPF=1')]='HASSPF=on'    
        if bool(pdata):
            # pprint(pdata)
            udata=conf.user_config(user)            
            if cl:
                udata['cloudlinux']={}
                udata['cloudlinux']['status']='on'        
            plan_maps=conf.user_map()
            for item in pdata:
                feature=item.split("=")
                # print(feature)
                try:
                    if str(feature[0]).startswith('lve_'):
                        udata['cloudlinux'][plan_maps[feature[0]]]= feature[1]
                    else:
                        udata[plan_maps[feature[0]]]=feature[1]
                except Exception:
                    # print(str(e))
                    pass
                # The cpanel featues and eenos features are  different.
            # log("Setting  Hosting feature name to default")
            udata['feature']='Default'
            correct_pan=get_pan_name(udata['package'],log)
            log("Setting Eenos hosting package name : %s" %correct_pan)
            udata['package']=correct_pan
            main_domain=udata['domain']
            log("Finding main domain : %s" %main_domain)
            udata['dedicatedip']=get_ips(backup,main_domain,log)
            udata['homedir']=get_homedir(user,backup,main_domain,log)
            log("Finding HomeDir : %s" %udata['homedir'])
            udata['nameservers']=get_nameserver(user,backup,main_domain,log)            
            log("Collecting name servers : %s" %','.join(udata['nameservers']))
            if str(udata['bandwidth']).isnumeric():
                log("Setting bandwidth properly %s bits" %udata['bandwidth'])
                newbw=convert_bw_to_eenos(int(str(udata['bandwidth'])),log)
                udata['bandwidth']=newbw
            if str(udata['quota']).isnumeric():
                log("Setting Disk Quota properly %s bytes" %udata['quota'])
                newq=convert_quota_to_eenos(int(str(udata['quota'])),log)
                udata['quota']=newq
            log("Checking if it is reseller account ..")
            isreseller=is_reseller(user,backup,log)
            selfown="off"
            if isreseller:
                udata['reseller']['status']="on"
                log("This account is reseller")
                log("Checking for self own account .. ")
                if udata['user'] == udata['owner']:
                    log("This user own his own account")
                    selfown="on"
                else:
                    log("This user doesn't own his account")
                    selfown="off"
                udata['reseller']['selfown']=selfown
            else:
                udata['reseller']['status']="off"
                udata['reseller']['selfown']="off"
                log("This is not a reseller account ..")
            if bool(data['skipresellerconfig']):
                log("You chose to skip reseller privileges ,, so disabing reseller privileges")
                udata['reseller']['status']="off"
                udata['reseller']['selfown']="off"            
            if  not bool(udata['contactemail']):                
                nco=udata['user']+'@'+udata['domain']
                log("Cpanel account doesn't have a contact email, settings a default one , %s" %nco)
                udata['contactemail']=nco
            isconverted=True
        else:
           udata=conf.user_config(user) 
    else:
        udata=conf.user_config(user)    
    if isconverted:
        return udata
    else:
        return dict()

def get_user_config(backup,workdir,user,data,log):
    ucfdir=workdir+'/userconfig'
    userdata=convert_cp_userdata(user,backup,data,log)
    uconfig=ucfdir+'/'+user
    try:
        os.makedirs(ucfdir)        
    except Exception as e:
        log(str(e))
        pass
    try:
        tools.writeyaml(uconfig,userdata,log)        
        log("Converting Eenos user configuration : %s" %uconfig)
    except Exception as e:
        log(str(e))
        pass

def get_reseller_conf(userdata,backup,workdir,user,log):    
    rdata=conf.default_reseller_conf(reseller=user)
    rcdir=workdir+'/resellerconfig'
    rlimitfile=backup+'/resellerconfig/my_reseller-limits.yaml'    
    if os.path.isfile(rlimitfile) and os.stat(rlimitfile).st_size >0:
        rlimit=tools.readyaml(rlimitfile,log)
        # pprint(rlimit)        
    else:
        rlimit=dict()
    if bool(rlimit) and user in rlimit and 'limits' in rlimit[user] and bool(rlimit[user]['limits']):
        userlimits=rlimit[user]['limits']
        if 'number_of_accounts' in userlimits:
            if 'accounts' in userlimits['number_of_accounts'] and bool(userlimits['number_of_accounts']['accounts']):
                reseller_limt=str(userlimits['number_of_accounts']['accounts'])
            else:
                reseller_limt='unlimited'
        else:
            reseller_limt='unlimited'
        rdata['reseller_limt'] =reseller_limt
        if 'resources' in  userlimits and 'type' in userlimits['resources'] and bool(userlimits['resources']['type']):
            resource=userlimits['resources']['type']
            if 'bw' in resource and bool(resource['bw']):
                reseller_bandwidth=str(resource['bw'])
            else:
                reseller_bandwidth='unlimited'
            if 'disk' in resource and bool(resource['disk']):
                reseller_quota = str(resource['disk'])
            else:
                reseller_quota ='unlimited'
            rdata['reseller_bandwidth'] =    reseller_bandwidth
            rdata['reseller_quota'] = reseller_quota        
    nsfile=backup+'/resellerconfig/resellers-nameservers'
    nss=userdata['nameservers']
    if os.path.isfile(nsfile) and os.stat(nsfile).st_size >0 :        
        nsdata=tools.readplain(nsfile,log)        
        if bool(nsdata):
            for item in nsdata:
                if str(item).startswith(user+':'):
                    na=list(filter(None,(item.split(':')[-1].split(','))))                    
                    if bool(na):
                        nss=na
        else:
            nss=userdata['nameservers']              
    else:
        nss=userdata['nameservers'] 
    rdata['nameservers']=nss
    rdata['contactemail']=userdata['contactemail']
    rdata['ips']['ipv4']=userdata['dedicatedip']['ipv4']
    rdata['ips']['ipv6']=userdata['dedicatedip']['ipv6']
    accounts=list()
    if userdata['user']==userdata['owner']:
        accounts.append(user)
    rdata['accounts']=accounts
    result=False
    try:
        if not os.path.isdir(rcdir):
            log("Creating reseller config dir")
            os.makedirs(rcdir)
    except Exception as e:
        log(str(e))
        pass
    rcofile=rcdir+'/'+user
    try:
        log("Creating reseller config  .. ")
        tools.writeyaml(rcofile,rdata,log)
        result=True
    except Exception as e:
        log(str(e))
        pass        
    return result

def get_reseller_packages(backup,workdir,user,log):
    log("Converting Reseller packages ")
    rcpkgdir=workdir+'/resellerpackages'
    try:
        if not os.path.isdir(rcpkgdir):
            os.makedirs(rcpkgdir)
    except Exception as e:
        log(str(e))
        pass
    cppkg_dir=backup+'/resellerpackages'
    packages={}
    if os.path.isdir(cppkg_dir):
        pfiles=glob.glob(cppkg_dir+'/*')
        pfiles.sort()
        if bool(pfiles):
            for pfile in pfiles:
                if os.path.isfile(pfile): 
                    pname=pfile.split('/')[-1]                              
                    if bool(re.search(r"[\s.]+",pname)):                        
                        plan_name=tools.convert_package_feature_name(pname,log)
                        log("Changing Cpanel Package Name from [ %s ] to [ %s ]" %(pname,plan_name))
                    else:
                        plan_name=pname
                    log("Converting cpanel Package : %s" %plan_name)
                    packages[plan_name]=pkgconvert.convert_cp(pfile,log,plan_name,owner=user)
    if bool(packages):
        for pname in packages:
            pfile=rcpkgdir+'/'+pname+'.yaml'            
            log("[ %s ] Creating Eenos reseller package : %s" %(user,pname))
            tools.writeyaml(pfile,packages[pname],log)
        # pprint(packages)

def get_reseller_features(workdir,log):
    rcfdir=workdir+'/resellerfeatures'
    try:
        if not os.path.isdir(rcfdir):
            log("Creating reseller feature directory")
            os.makedirs(rcfdir)
    except Exception as e:
        log(str(e))
        pass

def create_userdata_dirs(workdir,log):
    log("Creating Eenos userdata dirs ...")
    ucdir=workdir+'/userdata'
    if not os.path.isdir(ucdir):
        try:
            os.makedirs(ucdir)
        except Exception as e:
            log(str(e))
            pass
    thisdir=ucdir+'/mysql'
    if not os.path.isdir(thisdir):
        try:
            log("Creating mysql dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass
    thisdir=ucdir+'/vhosts/httpd'
    if not os.path.isdir(thisdir):
        try:
            log("Creating httpd dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/vhosts/nginx'
    if not os.path.isdir(thisdir):
        try:
            log("Creating nginx dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass   
    thisdir=ucdir+'/pgsql'
    if not os.path.isdir(thisdir):
        try:
            log("Creating pgsql dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/dns'
    if not os.path.isdir(thisdir):
        try:
            log("Creating dns dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/domains'
    if not os.path.isdir(thisdir):
        try:
            log("Creating domains dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/ssl'
    if not os.path.isdir(thisdir):
        try:
            log("Creating ssl dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/dovecot'
    if not os.path.isdir(thisdir):
        try:
            log("Creating dovecot dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass
    # Mail folders 
    thisdir=ucdir+'/mail'
    if not os.path.isdir(thisdir):
        try:
            log("Creating mail dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    # Mail folders 
    thisdir=ucdir+'/mail/domainfilter'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Domainfilter dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass   
    thisdir=ucdir+'/mail/mailman'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Mailing list dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/accountfilter'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Account filter dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/forwarder'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Email Forwarder dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/distribute'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Distribution List dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass         
    thisdir=ucdir+'/mail/pipe'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Email piping dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    

def build_domain_data(domain,domcnf,user,udata,backup,log,cdomain=None):
    log("Collecting domain configuration for : %s" %domain)
    data=tools.readyaml(domcnf,log)
    # pprint(data)
    domdata={}
    if bool(data):
        serveradmin=data['serveradmin']   
        log( "[ %s ] - Server Admin  - %s" %(domain,serveradmin))     
        docroot=data['documentroot']
        log( "[ %s ] - Document root - %s" %(domain,docroot))
        ipv4=data['ip']
        log( "[ %s ] - IPv4  - %s" %(domain,ipv4))
        if 'ipv6' in data and bool(data['ipv6']):
            for ip in data['ipv6']:
                ipv6=ip
            log( "[ %s ] - IPv6  - %s" %(domain,ipv6))
        else:
            ipv6=None
        # On a second thought get rid of all useless alias on cpanel
        # if 'serveralias' in data and bool(data['serveralias']):
        #     serveralias=list(filter(None,data['serveralias'].split(" ")))            
        # else:
        #     serveralias=list()
        #     serveralias.append('www.'+domain)
        serveralias=list()
        serveralias.append('www.'+domain)
        if domain in serveralias:
            serveralias.remove(domain)
        log( "[ %s ] - Server Aliases  - %s" %(domain,','.join(serveralias)))
        domdata=conf.domain_conf(domain,serveradmin,user,docroot,ipv4,ipv6,alias=serveralias)
        if cdomain:
            fpmconf=backup+'/userdata/'+cdomain+'.php-fpm.yaml'
        else:
            fpmconf=backup+'/userdata/'+domain+'.php-fpm.yaml'
        if os.path.isfile(fpmconf):
            domdata['phpfpm']='on'
        log( "[ %s ] - PHP-FPM  - %s" %(domain,domdata['phpfpm']))
        if 'phpversion' in data and bool(data['phpversion']) and str(data['phpversion']).__contains__("-"):
            cpversion=str(data['phpversion']).split('-')[-1]
            ephp='eenos-'+cpversion
            domdata['php']=ephp
        log( "[ %s ] - PHP Version  - %s" %(domain,domdata['php']))
    # pprint(domdata)
    return domdata

def  extract_cp_zone(zonefile,domain,log):
    log("Collecting dns from cpanel zone : %s" %domain)
    # log(zonefile)
    try:
        z=dns.zone.from_file(zonefile,domain)
    except Exception as e:
        log(str(e))
        z=None
        pass
    data=list()
    get_rid_of_cpanel=['whm','cpanel','webdisk','webmail','default._domainkey','cpcontacts','cpcalendars','_cpanel-dcv-test-record']
    if z:
        for (name, rdataset) in z.iterate_rdatasets():
            # print(rdataset.rdtype)
            # print(rdataset.rdtype.name)
            #     continue
            try:
                zname=str(name)
                zdata=str(rdataset)
                xdata={}
                if rdataset.rdtype == RdataType.SOA:            
                    continue
                elif rdataset.rdtype == RdataType.NS :
                    rans=list(filter(None,str(rdataset).split("\n")))
                    for ns in rans:
                        xdata={}
                        xdata['@']=ns
                        data.append(xdata)            
                    # # pprint(dir(rdataset))
                    # print(str(rdataset).split("\n"))
                    # print("-=========")
                elif rdataset.rdtype == RdataType.MX :
                    rans=list(filter(None,str(rdataset).split("\n")))                    
                    for mx in rans:
                        xdata={}  
                        if  str(mx).endswith('@'):
                            xdata['@']=mx
                        elif not str(mx).endswith('.'):
                            mx=mx+'.'+domain+'.' 
                            xdata['@']=mx
                        else:
                            xdata['@']=mx                                              
                        data.append(xdata) 
                elif rdataset.rdtype == RdataType.TXT and zname =='@':
                    rans=list(filter(None,str(rdataset).split("\n")))                    
                    for tx in rans:
                        xdata={}                        
                        xdata['@']=tx
                        data.append(xdata)
                elif not zname.startswith(tuple(get_rid_of_cpanel)):
                    # print("%s - %s " %(zname,zdata)) 
                    xdata={}
                    xdata[zname]= zdata 
                    data.append(xdata)
            except Exception as e:
                log(str(e))  
                pass
    if bool(data):
        return data
    else:
        return list()
    # pprint(data)

def create_dns_file(domain,workdir,domain_data,userdata,backup,log):
    zonefile=workdir+'/userdata/dns/'+domain
    cpzonefile=backup+'/dnszones/'+domain+'.db'
    if os.path.isfile(cpzonefile) and os.stat(cpzonefile).st_size > 0:
        cp_records=extract_cp_zone(cpzonefile,domain,log)
    else:
        cp_records=list()
    if 'ips' in domain_data and 'ipv4' in domain_data['ips'] and bool(domain_data['ips']['ipv4']):
        ipv4=domain_data['ips']['ipv4']
    else:
        ipv4=None
    if 'ips' in domain_data and 'ipv6' in domain_data['ips'] and bool(domain_data['ips']['ipv6']):
        ipv6=domain_data['ips']['ipv6']
    else:
        ipv6=None
    NS=userdata['nameservers']
    MX=dict()
    MX['0']=domain
    zone=conf.dns_conf(domain,ipv4,ipv6,NS,MX)
    nss=list()
    mxs={}
    arecords={}
    aaaarecords={}
    cnames={}
    gtxt=list()
    txts={}
    spfcustom=None
    if bool(cp_records):        
        for record in cp_records:
            rcdata=list(record.items())[0]
            rc_name=rcdata[0]
            rc_val=rcdata[-1]
            rc_type=rc_val.split(" ")
            record_type=rc_type[2]
            if record_type == 'NS': 
                nsname=str(rc_type[3])
                if nsname.endswith('.'):
                   nsname=nsname[:-1]      
                nss.append(nsname)
            if record_type == 'MX':
                mxtarget=rc_type[4]
                if mxtarget=='@':
                   mxtarget=domain
                elif  mxtarget.endswith('.'):
                    mxtarget=mxtarget[:-1]                
                mxs[rc_type[3]]=mxtarget
            if record_type == 'A':
                if rc_name == '@':
                    zone['ips']['ipv4']=rc_type[3]
                    zone['spf']['ipv4'].append(rc_type[3])                    
                else:
                    arecords[rc_name]={}
                    arecords[rc_name]['ip']=rc_type[3]
                    arecords[rc_name]['ttl']=rc_type[0]
            if record_type == 'AAAA':
                if rc_name == '@':
                    zone['ips']['ipv6']=ipaddress.ip_address(rc_type[3]).exploded
                    zone['spf']['ipv6'].append(ipaddress.ip_address(rc_type[3]).exploded)
                else:
                    aaaarecords[rc_name]={}
                    aaaarecords[rc_name]['ip']=ipaddress.ip_address(rc_type[3]).exploded  
                    aaaarecords[rc_name]['ttl']=rc_type[0]
            if record_type == 'CNAME':
                cntarget=rc_type[3]
                if rc_name != '@' and cntarget != '@':                    
                    if cntarget.endswith('.'):
                        cntarget=cntarget[:-1]
                    cnames[rc_name]={}
                    cnames[rc_name]['domain']=cntarget
                    cnames[rc_name]['ttl']=rc_type[0]
            if record_type == 'TXT':
                tdata=rc_val.split(" ",3)
                if rc_name == '@':
                    if 'spf' in tdata[3]:
                        if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                            tdata[3]=tdata[3][1:-1]                        
                        spfcustom=tdata[3]
                    else:
                        if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                            tdata[3]=tdata[3][1:-1]
                        gtxt.append(tdata[3])
                else:
                    if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                        tdata[3]=tdata[3][1:-1]
                    txts[rc_name]={}
                    txts[rc_name]['value']=tdata[3]
                    txts[rc_name]['ttl']=tdata[0]
                # pprint(tdata)
        try:
            # Remove duplicates
            zone['spf']['ipv4']=list(set(zone['spf']['ipv4'])) 
            zone['spf']['ipv6']=list(set(zone['spf']['ipv6'])) 
        except Exception as e:
            log(str(e))
            pass            
    if bool(nss):        
        zone['NS']=nss
    if bool(mxs):
        zone['MX']=mxs        
    if bool(arecords):
        zone['A']=arecords
    if bool(aaaarecords):
        zone['AAAA']=aaaarecords
    if bool(cnames):
        zone['CNAME']=cnames
    if bool(gtxt):
        zone['GTXT']=gtxt
    if bool(txts):
        zone['TXT']=txts
    if bool(spfcustom):
        zone['spfcustom']=spfcustom        
    if bool(zone):
        return zone
    else:
        return dict()
    
def update_subdomain_zone(subdomain,maindomain,name,sddon_data,workdir,backup,log):
    log("Adding subdomain to DNS Zone  subdomain : %s , main domain : %s , name : %s" %(subdomain,maindomain,name))
    zonefile=workdir+'/userdata/dns/'+maindomain
    # print(zonefile)
    # pprint(sddon_data)
    if os.path.isfile(zonefile) and os.stat(zonefile).st_size > 0:
        zdata=tools.readyaml(zonefile,log)
        # pprint(zdata)
        if name in zdata['A']:           
            del zdata['A'][name]
        if 'www.'+name in zdata['A']:        
            del zdata['A']['www.'+name]
        if name in zdata['AAAA']:
            del zdata['AAAA'][name]
        if 'www.'+name in zdata['AAAA']:
            del zdata['AAAA']['www.'+name]
        zdata['sub'][subdomain]={}
        zdata['sub'][subdomain]['name']=name
        zdata['sub'][subdomain]['ttl']='14400'
        if 'ips' in sddon_data and 'ipv4' in sddon_data['ips'] and bool(sddon_data['ips']['ipv4']):
            zdata['sub'][subdomain]['ipv4']=sddon_data['ips']['ipv4']
        if 'ips' in sddon_data and 'ipv6' in sddon_data['ips'] and bool(sddon_data['ips']['ipv6']):
            zdata['sub'][subdomain]['ipv6']=sddon_data['ips']['ipv6']
        tools.writeyaml(zonefile,zdata,log)
        # pprint(zdata)
    else:
        log("== Main domain zone data file not found , skippings dns of %s" %subdomain)
        # pprint(zdata)

def collect_domains(userdata,backup,workdir,user,log):
    main={}
    main_domain=userdata['domain']
    main['main_domain']=main_domain
    domcnf=backup+'/userdata/'+main_domain
    # print(domcnf)
    main_data=build_domain_data(main_domain,domcnf,user,userdata,backup,log)
    dfile=workdir+'/userdata/domains/'+main['main_domain']
    try:
        log("Creating Main domain configuration : %s" %main['main_domain'])
        tools.writeyaml(dfile,main_data,log)
        zdata=create_dns_file(main_domain,workdir,main_data,userdata,backup,log)
        if bool(zdata):
            zfile=workdir+'/userdata/dns/'+main_domain
            log("Creating DNS zone data of %s" %main_domain)
            tools.writeyaml(zfile,zdata,log)
    except Exception as e:
        log(str(e))
        pass
    
    # Addon domains 
    main['addon_domains']=list()
    afile=backup+'/addons'    
    cpaddons={}
    if os.path.isfile(afile) and os.stat(afile).st_size > 0:
        add_a=tools.readplain(afile,log)
        for cpadd in add_a:
            if '=' in cpadd:
                aspli=cpadd.split('=')
                adomin=aspli[0]
                cdom=str(aspli[-1]).replace('_','.')
                cpaddons[adomin]=cdom
    if bool(cpaddons):
        # pprint(cpaddons)
        for addon in cpaddons:
            dfile=backup+'/userdata/'+cpaddons[addon]
            addon_data=build_domain_data(addon,dfile,user,userdata,backup,log,cdomain=cpaddons[addon])
            domfile=workdir+'/userdata/domains/'+addon
            try:
                log("Creating Addon domain configuration : %s" %addon)
                tools.writeyaml(domfile,addon_data,log)
                main['addon_domains'].append(addon)
                zdata=create_dns_file(addon,workdir,addon_data,userdata,backup,log)
                if bool(zdata):
                    zfile=workdir+'/userdata/dns/'+addon
                    log("Creating DNS zone data of %s" %addon)
                    tools.writeyaml(zfile,zdata,log)
            except Exception as e:
                log(str(e))
                pass
            # pprint(addon_data)    
    # Parked domains
    main['parked_domains']=list()
    pafile=backup+'/pds' 
    if os.path.isfile(pafile) and os.stat(pafile).st_size > 0:
        pdd_a=tools.readplain(pafile,log)
        parkeddomain=list(filter(None,pdd_a))
        parkeddomain.sort()
        if bool(parkeddomain):
            dfile=workdir+'/userdata/domains/'+main_domain
            pdata=tools.readyaml(dfile,log)
            for pdom in parkeddomain:
                pdom_data=pdata
                pdom_data['servername']=pdom
                pdom_data['serveralias']=list()
                pdom_data['serveralias'].append('www.'+pdom)
                pdom_data['serveradmin']='webmaster@'+pdom
                try:
                    domfile=workdir+'/userdata/domains/'+pdom
                    log("Creating Parked domain configuration : %s" %pdom)
                    tools.writeyaml(domfile,pdom_data,log)
                    main['parked_domains'].append(pdom)
                    zdata=create_dns_file(pdom,workdir,pdom_data,userdata,backup,log)
                    if bool(zdata):
                        zfile=workdir+'/userdata/dns/'+pdom
                        log("Creating DNS zone data of %s" %pdom)
                        tools.writeyaml(zfile,zdata,log)
                except Exception as e:
                    log(str(e))
                    pass    
    # Sub domains
    main['sub_domains']={}
    sdfile=backup+'/sds'
    if os.path.isfile(sdfile) and os.stat(sdfile).st_size > 0:
        sdd_a=tools.readplain(sdfile,log)
        sdraw=list(filter(None,sdd_a))
        sdraw.sort()
        subdomains={}
        if bool(sdraw):
            for rsd in sdraw:
                rsdata=rsd.split('_')
                sname=rsdata[0]
                sof=rsdata[-1]
                sdom=str(rsd).replace('_','.')
                subdomains[sdom]={}
                subdomains[sdom]['name']=sname
                subdomains[sdom]['of']=sof
        if bool(subdomains):
            # pprint(subdomains)
            for subdomain in subdomains:
                dfile=backup+'/userdata/'+subdomain
                sddon_data=build_domain_data(subdomain,dfile,user,userdata,backup,log)
                domfile=workdir+'/userdata/domains/'+subdomain
                try:
                    log("Creating Sub domain configuration : %s" %subdomain)
                    tools.writeyaml(domfile,sddon_data,log)
                    main['sub_domains'][subdomain]=subdomains[subdomain] 
                    update_subdomain_zone(subdomain,subdomains[subdomain]['of'],subdomains[subdomain]['name'],sddon_data,workdir,backup,log)               
                except Exception as e:
                    log(str(e))
                    pass
    mfile=workdir+'/userdata/main'
    try:
        log("Creating main data file ..")
        tools.writeyaml(mfile,main,log)
    except Exception as e:
        log(str(e))
        pass    
    # pprint(main)

def colloect_ftp_accounts(backup,workdir,log):
    log("collecting FTP Accounts ")
    cpftp=backup+'/proftpdpasswd'
    eftp=workdir+'/userdata/.ftp.passwd'
    if os.path.isfile(cpftp) and os.stat(cpftp).st_size >0 :
        cpftpacc=tools.readplain(cpftp,log)
        for acc in cpftpacc:
            acount=acc.split(':')[0]
            if not '@' in acount:
               cpftpacc.remove(acc) 
        if bool(cpftpacc):
            try:
                tools.writeplain(eftp,cpftpacc,log)
                security.safe_chmod(eftp,0o640)
            except Exception as e:
                log(str(e))
                pass
        else:
            log("Not ftp accounts found")
    else:
        log("Not ftp accounts found")

def change_cron_shell(cpcronf,log):
    if os.path.isfile(cpcronf):
        cdata=tools.readplain(cpcronf,log)
        if bool(cdata):            
            doupdate=False
            for line in cdata:
                if str(line).startswith("SHELL="):
                    cdata[cdata.index(line)]='SHELL="/usr/local/eenos/bin/jailshell"'
                    doupdate=True
            if doupdate:
                tools.writeplain(cpcronf,cdata,log) 

def collect_cron_jobs(backup,workdir,user,log):
    log("Collecting Cpanel cron jobs")
    cpcronf=backup+'/cron/'+user    
    if os.path.isfile(cpcronf) and os.stat(cpcronf).st_size > 0:
        change_cron_shell(cpcronf,log)
        cpcrons=tools.readplain(cpcronf,log)
        cpcrons=list(filter(None,cpcrons))
        crontab={}
        if bool(cpcrons):
            for cron in cpcrons:
                if cron.startswith('MAILTO='):
                    cmail=cron.split("=")[-1]
                    if cmail.startswith('"') and cmail.endswith('"'):
                        cmail=cmail[1:-1]
                    crontab['mailto']=cmail
                elif cron.startswith('SHELL='):
                    continue
                else:                    
                    try:                        
                        cdata=cron.split(" ",5)
                        ccommand=str(cdata[5])                    
                        cschedule=' '.join(cdata[0:5:1])
                        cronid=tools.getid()                        
                        crontab[cronid]={}
                        crontab[cronid]['command']=ccommand
                        crontab[cronid]['schedule']=cschedule
                    except Exception as e:
                        log(str(e))
                        pass        
        if bool(crontab):
            ecron=workdir+'/userdata/crontab'            
            try:
                log("Creating Eenos cron tab  data file ..")
                tools.writeyaml(ecron,crontab,log)
            except Exception as e:
                log(str(e))
                pass
            # pprint(crontab)
        else:
            log("No cron jbos foud ..")
        ecrdir=workdir+'/cron'
        try:
            log("Copying Crontab File ..")
            if not os.path.isdir(ecrdir):
                os.makedirs(ecrdir)
            shutil.copy(cpcronf,ecrdir+"/"+user)
        except Exception as e:
            log(str(e))
            pass
    else:
        log("No cron jobs found ..")

def collect_mail_filter_forwardor_pip(backup,workdir,data,log):
    if bool(data['skipemail']):
        log("Skipping email forwarders , email pips and email filters")
    else:
        cpvfdir=backup+'/vf'
        edomfilterdir=workdir+'/userdata/mail/domainfilter'
        if os.path.isdir(cpvfdir):
            cpvfs=glob.glob(cpvfdir+'/*')
            cpvfs.sort()
            for f in cpvfs:
                if os.path.isfile(f) and os.stat(f).st_size>0:
                    vdom=f.split('/')[-1]
                    log("Copying Exim domain SMTP filter %s" %vdom)
                    try:
                        dest=edomfilterdir+'/'+vdom
                        shutil.copy(f,dest)
                    except Exception as e:
                        log(str(e))
                        pass
        cpva=backup+'/va'
        forworders={}
        mailpips={}
        if os.path.isdir(cpva):
            cpvafs=glob.glob(cpva+'/*')
            cpvafs.sort()
            for f in cpvafs:
                if os.path.isfile(f) and os.stat(f).st_size>0:
                    vdom=f.split('/')[-1]
                    log("Collecting Email Forwarders and Pips - %s" %vdom)
                    try:
                        fdata=tools.readplain(f,log)
                        # Wed don't need wild card catch to fill inbox with spam
                        fdata=[x for x in fdata if not x.startswith('*:')]
                        fdata=list(filter(None,fdata))
                        if bool(fdata):
                            forworders[vdom]={}
                            mailpips[vdom]={}
                            for item in fdata:
                                item_data=item.split(":",1)
                                if str(item_data[1]).__contains__("|"):
                                    mailpips[vdom][item_data[0]]=item_data[1].strip()
                                else:
                                    forworders[vdom][item_data[0]]=item_data[1].strip()
                                # pprint(item_data)
                            # pprint(fdata)
                    except Exception as e:
                        log(str(e))
                        pass
        eenos_forward_dir=workdir+'/userdata/mail/forwarder'
        eenos_pip_dir=workdir+'/userdata/mail/pipe'
        if bool(forworders):
            for domain in forworders:
                if bool(forworders[domain]):
                    try:
                        ffile=eenos_forward_dir+'/'+domain
                        log("Creating Eenos forwarder on %s" %domain)
                        tools.writeyaml(ffile,forworders[domain],log)
                    except Exception as e:
                        log(str(e))
                        pass
        if bool(mailpips):
            for domain in mailpips:
                if bool(mailpips[domain]):
                    try:
                        ffile=eenos_pip_dir+'/'+domain
                        log("Creating Eenos Mail pipes on %s" %domain)
                        tools.writeyaml(ffile,mailpips[domain],log)
                    except Exception as e:
                        log(str(e))
                        pass
        cphome=backup+'/homedir'
        acfdir=workdir+'/userdata/mail/accountfilter'
        if os.path.isdir(cphome):
            cpfilters=glob.glob(cphome+'/etc/*/*/filter')
            cpfilters.sort()
            if bool(cpfilters):
                log("Collecting Cpanel  Email Account filters")
                for f in cpfilters:
                    fdata=f.split("/")
                    domain=fdata[-3]
                    mail=fdata[-2]
                    try:
                        log("Create Email account filer for %s@%s" %(mail,domain))
                        fdir=acfdir+'/'+domain
                        if not os.path.isdir(fdir):
                            os.makedirs(fdir)
                        shutil.copy(f,fdir+'/'+mail)
                    except Exception as e:
                        log(str(e))
                        pass
                    
        # pprint(forworders)
        # pprint(mailpips)

def collect_ssl_crts(backup,workdir,log):    
    ssldir=workdir+'/userdata/ssl'
    cpssldir=backup+'/apache_tls'
    cpsfiles=list()
    if os.path.isdir(cpssldir):
        cpsfile=glob.glob(cpssldir+'/*')
        if bool(cpsfile):
            for cf in cpsfile:
                if os.path.isfile(cf) and os.stat(cf).st_size >0:
                    cpsfiles.append(cf)
    if bool(cpsfiles):
        log("Collecting Cpanel SSL Certificates ..")
        for crt in cpsfiles:        
            try:
                domain=None
                cert_dict = ssl._ssl._test_decode_cert(crt)
                if bool(cert_dict['subject'][0]):
                    namedata=dict((x, y) for x, y in cert_dict['subject'][0])
                    if 'commonName' in namedata:
                        domain=namedata['commonName']                        
                    else:
                        domain=None
                else:
                    domain=None
            except Exception as e:
                log(str(e))
                cert_dict=dict()
                domain=None
                pass
            if domain:
                log("Certificate domain .. %s" %domain)
                try:
                    with open(crt,'r') as r:
                        certificate=r.read()
                except Exception as e:
                    log(str(e))
                    certificate=None
                    pass
                if bool(certificate):
                    # print(crt)
                    if str(certificate).__contains__('BEGIN RSA'):
                        private=re.search(r"(?<=-----BEGIN RSA PRIVATE KEY-----).*?(?=-----END RSA PRIVATE KEY-----)", certificate, flags=re.DOTALL)
                        pstart='-----BEGIN RSA PRIVATE KEY-----'
                        pend='-----END RSA PRIVATE KEY-----\n'
                    elif str(certificate).__contains__('BEGIN DSA'):
                        private=re.search(r"(?<=-----BEGIN DSA PRIVATE KEY-----).*?(?=-----END DSA PRIVATE KEY-----)", certificate, flags=re.DOTALL)
                        pstart='-----BEGIN DSA PRIVATE KEY-----'
                        pend='-----END DSA PRIVATE KEY-----\n'
                    elif str(certificate).__contains__('BEGIN ANY'):
                        private=re.search(r"(?<=-----BEGIN ANY PRIVATE KEY-----).*?(?=-----END ANY PRIVATE KEY-----)", certificate, flags=re.DOTALL)
                        pstart='-----BEGIN ANY PRIVATE KEY-----'
                        pend='-----END ANY PRIVATE KEY-----\n'
                    elif str(certificate).__contains__('BEGIN EC'):
                        private=re.search(r"(?<=-----BEGIN EC PRIVATE KEY-----).*?(?=-----END EC PRIVATE KEY-----)", certificate, flags=re.DOTALL)
                        pstart='-----BEGIN EC PRIVATE KEY-----'
                        pend='-----END EC PRIVATE KEY-----\n'
                    else:
                        private = re.search(r"(?<=-----BEGIN PRIVATE KEY-----).*?(?=-----END PRIVATE KEY-----)", certificate, flags=re.DOTALL)
                        pstart='-----BEGIN PRIVATE KEY-----'
                        pend='-----END PRIVATE KEY-----\n'
                    if private:
                        # print( private.group() )
                        sdata=private.group()
                        # print(type(sdata))
                        # sprivate='-----BEGIN PRIVATE KEY-----'
                        sprivate=pstart
                        sprivate+=sdata
                        # sprivate+='-----END PRIVATE KEY-----'
                        sprivate+=pend.strip()                        
                        certificate=str(certificate).replace(pstart,'')
                        certificate=str(certificate).replace(sdata,'')
                        certificate=str(certificate).replace(pend,'')
                        # print("Extracted certificate ...")
                        # pprint(certificate)
                        # print("Extracted private Key")
                        # pprint(sprivate)                        
                        crt_file=ssldir+'/'+domain+'-crt.pem'
                        key_file=ssldir+'/'+domain+'-key.pem'
                        try:
                            log("[ %s ] Creating Eenos SSL certificate " %domain)
                            with open(crt_file , 'w') as c:
                                c.write(certificate)
                            log("[ %s ] Creating Eenos SSL Private Key " %domain)
                            with open(key_file,'w') as k:
                                k.write(sprivate)
                            security.safe_chmod(key_file,0o640)
                        except Exception as e:
                            log(str(e))
                            pass
                    else:
                        log("Unable to retrieve privte key skipping ssl of domain %s" %domain)
                else:
                    log("Unable to read certificate data of domain : %s" %domain)

def collect_domain_keys(backup,workdir,log):
    cpdkims=backup+'/domainkeys'
    enosdkims=workdir+'/domain_keys'
    if os.path.isdir(cpdkims):
        log("Copying Cpanel Domain Keys ..")
        try:
            shutil.copytree(cpdkims,enosdkims)
        except Exception as e:
            log(str(e))
            pass

def collect_hostname(backup,log):
    cphfile=backup+'/meta/hostname'
    hostname=socket.gethostname()
    if os.path.isfile(cphfile) and os.stat(cphfile).st_size > 0:
        try:
            with open(cphfile,'r') as h:
                hostname=h.read().strip()
            if bool(hostname):
                log("Found Cpanel Account Hostname : %s" %hostname)
        except Exception as e:
            log(str(e))
            hostname=socket.gethostname()
            pass
    return hostname
def collect_mysql_dbs(backup,workdir,log):
    cpmsqldir=backup+'/mysql'
    eenos_sqldir=workdir+'/mysql/dbs'
    databases=dict()
    if os.path.isdir(cpmsqldir):
        mfiles=glob.glob(cpmsqldir+'/*.sql')
        if bool(mfiles):
            log("Collecting MySQL Databases")
            for df in mfiles:
                sqlname=df.split('/')[-1]
                dbname=sqlname.split(".")[0]
                target=eenos_sqldir+'/'+sqlname
                try:
                    log("[ %s ] Copying MySQL database dump - %s" %(dbname,sqlname))
                    shutil.copy(df,target)
                    databases[dbname]={}
                    databases[dbname]['host']='localhost'
                    databases[dbname]['user']={}
                except Exception as e:
                    log(str(e))
                    pass
    if bool(databases):
        ddata=workdir+'/userdata/mysql/databases'
        try:
            log("Creating Eenos MySQL database list")
            tools.writeyaml(ddata,databases,log)
        except Exception as e:
            log(str(e))
            pass

def collect_mysql_users(backup,workdir,user,userdata,hostname,log):
    cpsql_user_file=backup+'/mysql.sql'
    ddata=workdir+'/userdata/mysql/databases'
    user_query_dir=workdir+'/mysql/users'
    if os.path.isfile(ddata):
        databases=tools.readyaml(ddata,log)
    else:
        databases=dict()
    if 'dedicatedip' in userdata and 'ipv4' in userdata['dedicatedip'] and bool(userdata['dedicatedip']['ipv4']):
        ip=userdata['dedicatedip']['ipv4']        
    else:
        ip='127.0.0.1'
    if os.path.isfile(cpsql_user_file) and os.stat(cpsql_user_file).st_size > 0:
        cpu_users=tools.readplain(cpsql_user_file,log)  
        for item in cpu_users:
            if not item.startswith('GRANT'):
                cpu_users.remove(item)
            elif str(item).__contains__('\\'):
                this=item.replace('\\',"")
                cpu_users[cpu_users.index(item)]=this
                # print(this)
        if bool(cpu_users):
            cpu_users.sort()
            for_delete=list()
            usersql={}
            useryaml={}
            for item in cpu_users:
                if str(item).startswith("GRANT ALL PRIVILEGES"):
                    pvdata=item.split("'")
                    if pvdata[1]==user:
                        # Just remove the cpanel user grant privillege we don't use it
                        for_delete.append(item)
                if str(item).startswith("GRANT USAGE ON"):
                    gvdata=item.split("'")
                    if gvdata[1]==user:
                        # Just remove the cpanel user grant privillege we don't use it
                        for_delete.append(item)
                    # pprint(gvdata)
            if bool(for_delete):
                for ditem in  for_delete:
                    cpu_users.remove(ditem)            
            # Replace hostname and IP with localhost
            for item in cpu_users:
                if str(item).startswith("GRANT ALL PRIVILEGES"):
                    try:
                        pvdata=item.split("'")
                        if str(pvdata[3]) == ip or str(pvdata[3]) == hostname:
                            newitem=str(item).replace(pvdata[3],'localhost')
                            cpu_users[cpu_users.index(item)]=newitem
                            # print( " old item : %s" %item)
                            # print(newitem)
                    except Exception as e:
                        log(str(e))                 
                        pass
                if str(item).startswith("GRANT USAGE ON"):
                    try:
                        pvdata=item.split("'")
                        # pprint(pvdata)
                        if str(pvdata[3]) == ip or str(pvdata[3]) == hostname:
                            newitem=str(item).replace(pvdata[3],'localhost')
                            cpu_users[cpu_users.index(item)]=newitem
                    except Exception as e:
                        log(str(e))                 
                        pass
            # Remove duplicates
            cpu_users=list(set(cpu_users))
            cpu_users.sort()            
            for item in cpu_users:
                if str(item).startswith("GRANT USAGE ON"):
                    try:
                        pvdata=item.split("'")
                        db_user=pvdata[1]
                        db_host=pvdata[3]
                        db_passwd=pvdata[5]
                        if not db_user in usersql:
                            usersql[db_user]={}
                            usersql[db_user]['password']=''
                            usersql[db_user]['querry']=list()                    
                        if pvdata[3]== 'localhost':
                            usersql[db_user]['password']=db_passwd
                            useryaml[db_user]={}
                            useryaml[db_user]['access_host']='localhost'
                            useryaml[db_user]['host']='localhost'                                                
                        usersql[db_user]['querry'].append(conf.mysql_user_create(mysql_user=db_user,password=db_passwd,host=db_host))
                        usersql[db_user]['querry'].append(conf.mysql_user_grant_usage(mysql_user=db_user,host=db_host))
                    except Exception as e:
                        log(str(e))                 
                        pass
            for item in cpu_users:
                if str(item).startswith("GRANT ALL PRIVILEGES"):
                    try:
                        pvdata=item.split("'")
                        dbdata=item.split("`")
                        db_user=pvdata[1]
                        db_name=dbdata[1]
                        if db_user in usersql:
                            usersql[db_user]['querry'].append(item) 
                        if db_name in  databases:
                            databases[db_name]['user'][db_user]={}
                            databases[db_name]['user'][db_user]['grant']='ALL' 
                    except Exception as e:
                        log(str(e))                 
                        pass
            try:
                log("Updating Eenos MySQL database ownerships")
                tools.writeyaml(ddata,databases,log)
            except Exception as e:
                log(str(e))
                pass
            ddatauser=workdir+'/userdata/mysql/users'
            try:
                log("Updating Eenos MySQL database users list")
                tools.writeyaml(ddatauser,useryaml,log)
            except Exception as e:
                log(str(e))
                pass
            for sqluser in usersql:
                if bool(usersql[sqluser]['password']):
                    thispass=usersql[sqluser]['password']
                else:
                    log("[ %s ] MySQL localhost password is missing , assignign random password" %sqluser)
                    thispass=tools.getid()
                # querry=conf.mysql_user_create(mysql_user=sqluser,password=thispass)
                querry=''
                if bool(usersql[sqluser]['querry']):
                    for q in usersql[sqluser]['querry']:
                        querry+=q+'\n'
                usql=user_query_dir+'/'+sqluser+'.sql'
                try:
                    log("[ %s ] Creating Eenos MySQL User create query file %s.sql " %(sqluser,sqluser))
                    with open(usql,'w') as ur:
                        ur.write(querry)                    
                except Exception as e:
                    log(str(e))
                    pass
                # print(querry)
        else:
            log("No MySQL database users availabel")
            
    

def collect_mysql_dbs_users(backup,workdir,user,userdata,hostname,log):
    log("Collecting Cpanel MySQL Databases and MySQL users")
    emy_user_dir=workdir+'/mysql/users'
    emy_sql_dir=workdir+'/mysql/dbs'
    try:
        if not os.path.isdir(emy_user_dir):
            log("Creating MySQL user directory")
            os.makedirs(emy_user_dir)
        if not os.path.isdir(emy_sql_dir):
            log("Creating MySQL DB directory")
            os.makedirs(emy_sql_dir)
    except Exception as e:
        log(str(e))
        pass
    collect_mysql_dbs(backup,workdir,log)
    collect_mysql_users(backup,workdir,user,userdata,hostname,log)

def collect_pgsql_dbs(backup,workdir,log):
    cppgsqldir=backup+'/psql'
    eenos_sqldir=workdir+'/pgsql/dbs'
    databases=dict()
    if os.path.isdir(cppgsqldir):
        mfiles=glob.glob(cppgsqldir+'/*.tar')
        if bool(mfiles):
            log("Collecting PostgreSQL Databases")
            for df in mfiles:
                sqlname=df.split('/')[-1]
                dbname=sqlname.split(".")[0]
                target=eenos_sqldir+'/'+dbname+'.sql'
                try:
                    log("Converting Cpanel dump format to SQL Dump file [ %s => %s.sql ]" %(sqlname,dbname))
                    cmd='pg_restore -c '+df+' > '+target
                    logcmd='$ pg_restore -c '+sqlname+ ' > '+dbname+'.sql'
                    # log(cmd)
                    log("[ %s ]" %logcmd)
                    commands.live(cmd,log)                    
                    if os.path.isfile(target):                    
                        databases[dbname]={}
                        databases[dbname]['host']='localhost'
                        databases[dbname]['user']={}
                    else:
                        log("Failed converting PgSQL database %s " %dbname)
                except Exception as e:
                    log(str(e))
                    pass
    if bool(databases):
        ddata=workdir+'/userdata/pgsql/databases'
        try:
            log("Creating Eenos PostgreSQL database list")
            tools.writeyaml(ddata,databases,log)
        except Exception as e:
            log(str(e))
            pass

def collect_pgsql_users(backup,workdir,user,log):
    cpsql_user_file=backup+'/psql_users.sql'
    cpgrant_user_file=backup+'/psql_grants.sql'
    ddata=workdir+'/userdata/pgsql/databases'
    user_query_dir=workdir+'/pgsql/users'
    if os.path.isfile(ddata):
        databases=tools.readyaml(ddata,log)
    else:
        databases=dict()
    cpu_users=list()
    deluser=list()
    cpu_grant=list()
    delgrant=list()
    if os.path.isfile(cpsql_user_file) and os.stat(cpsql_user_file).st_size > 0 :
        cpu_users_data=tools.readplain(cpsql_user_file,log) 
        for pusr in cpu_users_data:
            if str(pusr).startswith("CREATE"):
                cpu_users.append(pusr)
    if os.path.isfile(cpgrant_user_file) and os.stat(cpgrant_user_file).st_size > 0 :
        cpu_users_data=tools.readplain(cpgrant_user_file,log) 
        for pusr in cpu_users_data:
            if str(pusr).startswith("GRANT"):
                newitem=str(pusr).replace('GRANT','GRANT ALL PRIVILEGES ON DATABASE')
                cpu_grant.append(newitem)
    # Remove system user as we don't user it
    for item in cpu_users:
        idata=str(item).split('"')
        if idata[1] == user :
            deluser.append(item)
    if bool(deluser):
        for item in deluser:
            cpu_users.remove(item)

    for item in cpu_grant:
        idata=str(item).split('"')        
        if idata[3] == user :
            delgrant.append(item)
    if bool(delgrant):
        for item in delgrant:
            cpu_grant.remove(item)
    cpu_users=list(set(cpu_users))
    cpu_users.sort()
    cpu_grant=list(set(cpu_grant))
    cpu_grant.sort()
    usersql={}
    useryaml={}
    if bool(cpu_users):
        for item in cpu_users:
            try:
                pvdata=item.split('"')                
                db_user=pvdata[1]
                
                if not db_user in usersql:
                    usersql[db_user]={}                    
                    usersql[db_user]['querry']=list()
                if not db_user in useryaml:
                    useryaml[db_user]={}     
                    useryaml[db_user]['host']='localhost'                                                                   
                usersql[db_user]['querry'].append(item)                
            except Exception as e:
                log(str(e))                 
                pass
        if bool(cpu_grant):
            for item in cpu_grant:
                try:
                    pvdata=item.split('"')                
                    db_user=pvdata[3]
                    db_name=pvdata[1]
                    if db_user in usersql:
                        usersql[db_user]['querry'].append(item) 
                    if db_name in  databases:
                        databases[db_name]['user'][db_user]={}
                        databases[db_name]['user'][db_user]['grant']='ALL PRIVILEGES'
                except Exception as e:
                    log(str(e))                 
                    pass
        try:
            log("Updating Eenos PostgreSQL database ownerships")
            tools.writeyaml(ddata,databases,log)
        except Exception as e:
            log(str(e))
            pass
        ddatauser=workdir+'/userdata/pgsql/users'
        try:
            log("Updating Eenos PostgreSQL database users list")
            tools.writeyaml(ddatauser,useryaml,log)
        except Exception as e:
            log(str(e))
            pass
        for sqluser in usersql:            
            querry=''
            if bool(usersql[sqluser]['querry']):
                for q in usersql[sqluser]['querry']:
                    querry+=q+'\n'
            usql=user_query_dir+'/'+sqluser+'.sql'
            try:
                log("[ %s ] Creating Eenos PostgreSQL User create query file %s.sql " %(sqluser,sqluser))
                with open(usql,'w') as ur:
                    ur.write(querry)                    
            except Exception as e:
                log(str(e))
                pass
    else:
        log("No PostgreSQL users available .. ")   

def collect_pgql_dbs_users(backup,workdir,user,log):
    log("Collecting PostgreSQL Databases and Users")
    emy_user_dir=workdir+'/pgsql/users'
    emy_sql_dir=workdir+'/pgsql/dbs'
    try:
        if not os.path.isdir(emy_user_dir):
            log("Creating PostgreSQL user directory")
            os.makedirs(emy_user_dir)
        if not os.path.isdir(emy_sql_dir):
            log("Creating PostgreSQL DB directory")
            os.makedirs(emy_sql_dir)
    except Exception as e:
        log(str(e))
        pass
    collect_pgsql_dbs(backup,workdir,log)
    collect_pgsql_users(backup,workdir,user,log)

def collect_homedir(backup,workdir,user,data,log):    
    if bool(data['skiphomedir']):
        log("Skipping Home Directory")
    else:
        target=workdir+'/homedir'
        source=backup+'/homedir'
        if not os.path.isdir(target):
            log("Creating homedir")
            os.makedirs(target)
        if not os.path.isdir(source):
            log("Cpanel backup doesn't have homedir , skipping coping home directory")
        else:
            # pprint(data)
            if bool(data['skipemail']):
                log("Skipping Emails")
                exccludelist="'mail/*','.cagefs/','lscache/','.cpanel','cpanel3-skel','cache/','.cl.selector','.clwpos','.cphorde','.cpnginx','.cpremote','.softaculous','.subaccounts','.wp-toolkit*','access-logs','perl5','cpbackup-exclude.conf','.spamassassinboxenable','.spamassassinenable','tmp/*','logs/*','.razor','.trash'"
            else:
                exccludelist="'.cagefs/','lscache/','.cpanel','cpanel3-skel','cache/','.cl.selector','.clwpos','.cphorde','.cpnginx','.cpremote','.softaculous','.subaccounts','.wp-toolkit*','access-logs','perl5','cpbackup-exclude.conf','.spamassassinboxenable','.spamassassinenable','tmp/*','logs/*','.razor','.trash'"
            cmd="/usr/bin/rsync --safe-links --exclude={"+exccludelist+"} -avp "+source+"/ "+target+"/"            
            # log("[command : %s]" %cmd)
            log("Copying Contents of Home Directory ..")
            commands.live(cmd,log)
            log("Done")
def collect_suspended(backup,workdir,user,log):
    cpsuspend=backup+'/suspended/'+user
    esuspend=workdir+"/suspended"
    if os.path.isfile(cpsuspend):
        log("Adding suspened  details")
        reason='Converted suspended'
        try:
            if not os.path.isdir(esuspend):
                os.makedirs(esuspend)
            with open(cpsuspend, 'r') as sp:
                reason=sp.read().strip()
            esfile=esuspend+'/'+user
            sdata={}
            sdata['reason']=reason
            sdata['user']=user
            now=datetime.now()    
            times=now.strftime("%a, %d-%b-%Y %I:%M:%S, " )
            sdata['suspendtime']=str(times)   
            tools.writeyaml(esfile,sdata,log)
        except Exception as e:
            log(str(e))
            pass
        
def clean_tmp_bkp(backup,log):
    log("Cleaning temporary files .. ")
    try:
        shutil.rmtree(backup)
    except Exception as e:
        log(str(e))
        pass
            
def start(user,backup,data,log,live=True):
    workdir=data['myworkdir']+'/eenosbkp-'+user
    isworkdir=setup_work_dir(workdir,log)
    result=None
    if isworkdir:
        log('Workdir generated successfully  %s ' %workdir)
        try:
            edata={}
            edata['user']=user
            security.safe_chmod(workdir,0o700)
            get_shadow(backup,workdir,log)
            get_user_config(backup,workdir,user,data,log)
            eucf=workdir+'/userconfig/'+user
            userdata=tools.readyaml(eucf,log)
            if bool(userdata):
                havereseller=is_reseller(user,backup,log)
                if havereseller and not bool(data['skipresellerconfig']):
                    edata['reseller']='yes'
                    log("Collecting reseller settings .. ")
                    get_reseller_conf(userdata,backup,workdir,user,log)
                    get_reseller_packages(backup,workdir,user,log)
                    get_reseller_features(workdir,log)
                else:
                    edata['reseller']='no'
                hostname=collect_hostname(backup,log)
                create_userdata_dirs(workdir,log)
                collect_domains(userdata,backup,workdir,user,log)
                colloect_ftp_accounts(backup,workdir,log)
                collect_cron_jobs(backup,workdir,user,log)
                collect_mail_filter_forwardor_pip(backup,workdir,data,log)
                collect_ssl_crts(backup,workdir,log)
                collect_domain_keys(backup,workdir,log)
                if bool(data['skipmysql']):
                    log("Skipping MySQL Databases and Users")
                else:
                    collect_mysql_dbs_users(backup,workdir,user,userdata,hostname,log) 
                if bool(data['skippgsql']):
                    log("Skipping PostgreSQL databases and users")
                else:
                    collect_pgql_dbs_users(backup,workdir,user,log) 
                edata['host']=hostname
                edata['backuptime']= str(datetime.now())
                edata['homedir']=userdata['homedir']
                edata['disk']=str(userdata['homedir']).rsplit('/',1)[0]
                tools.writeyaml(workdir+'/.eenos',edata,log) 
                collect_homedir(backup,workdir,user,data,log)
                collect_suspended(backup,workdir,user,log)
                # Remove temporary backup dir
                clean_tmp_bkp(backup,log)
                result=workdir
            else:
                log("Eenos user configuration generation failed")
                log("Cpanel to Eenos conversion failed")
        except Exception as e:
            log(str(e))
            result=None
            pass
    else:
        log('Unable to create workdir , cpanel conversion failed for account : %s' %user)
        result=None
    # pprint(data)
    return result
       
