import os
import shutil
from pprint import pprint
import glob
import re
from cptoeenos import tools,commands,cpanel,tarops,pkgconvert

def get_cpuser():
    cpdir='/var/cpanel/users'
    accounts=list()
    if os.path.isdir(cpdir):
        ufiles=glob.glob(cpdir+'/*')
        if bool(ufiles):
            for u in ufiles:
                if os.path.isfile(u) and os.stat(u).st_size>0:
                    accounts.append(str(u).split('/')[-1])
            accounts.sort()
    return accounts

def convert(choice,log):
    log("--------------------------------------")
    log(" Full Cpanel to Eenos Migration ")
    log("--------------------------------------")
    basedir=choice['myworkdir']
    cpusers=get_cpuser()    
    uploaded_backups={}
    uploaded_packages=pkgconvert.convert(choice,log)    
    for cpuser in cpusers:
        log("Migrating Cpanel Account : %s" %cpuser)
        workdir=basedir+'/cpmove-'+cpuser 
        result=None
        try:
            if os.path.isdir(workdir):
                log("Cleaning work directory : %s" %workdir)
                shutil.rmtree(workdir)
        except Exception as e:
            log(str(e))
            pass
        cp_cmd=tools.get_pkgacct_cmd(choice)
        cmd=cp_cmd+' '+cpuser+' '+basedir    
        try:
            log(cmd)
            commands.live(cmd,log)
            cresult=workdir
        except Exception as e:
            log(str(e))
            cresult=False
            pass
        if cresult and os.path.isdir(cresult) and os.path.isfile(cresult+'/cp/'+cpuser):
            log("Cpanel backup generated successfully  : %s" %cresult)
            log("Proceeding with backup conversion")
            backup=cpanel.start(cpuser,cresult,choice,log)
            if backup:
                outputfile=backup+'.tar.gz'
                archfile=tarops.make_gzip_tar_from_dir(outputfile,backup,log)
                if archfile:
                    pkgname=str(archfile).split('/')[-1]
                    log("Cpanel account converted successfully  : %s " %pkgname)
                    uploadpath=str(choice['saveto'])+'/'+pkgname
                    try:
                        log("Uploading Eenos backup to : %s" %uploadpath)
                        shutil.copy(archfile,uploadpath)
                        result=uploadpath
                    except Exception as e:
                        log(str(e))
                        result=None
                        pass
                    try:
                        log("Cleaning temporary backup files .. %s" %backup)                    
                        shutil.rmtree(backup)
                        os.remove(archfile)
                    except Exception as e:
                        log(str(e))
                        pass
                    if result:
                        log("===================================")
                        log("Your Eenos Backup is : %s" %result)
                        hashes=tools.get_hash(result,log,result=True)
                        uploaded_backups[cpuser]={}
                        uploaded_backups[cpuser]['file']=result
                        uploaded_backups[cpuser]['md5']=hashes['md5']
                        uploaded_backups[cpuser]['sha1']=hashes['sha1']
                    else:
                        log("Cpanel to Eenos conversion failed for account : %s" %cpuser)                    
                else:
                    log("Unable to create archive backup file , backup failed  cpuser - %s" %cpuser)
                    result=None
            else:
                log("Cpanel to Eenos account conversion failed , Cpanel account : %s" %cpuser) 
        else:
            log("Cpanel backup generation failed. Unable to converet backup to Eenos : %s " %cpuser)    
    log("Done.")    
    if bool(uploaded_backups) or bool(uploaded_packages):
        log("............................. SUMMARY .............................")
        if bool(uploaded_packages):
            log("========== Hosting Packages ===========")
            log("Converted Hosting Packages : %s" %uploaded_packages)
        if bool(uploaded_backups):
            log("========== Hosting Accounts ===========")
            log("Cpanel User".ljust(20)+"Eenos Backup".ljust(55)+"MD5 Check Sum".ljust(35)+"SHA1 Check Sum")  
            # log("You can Upload the following converted cpanel accounts to Eenos")
            for item in uploaded_backups:
                log("%s %s %s %s" %(item.ljust(19),uploaded_backups[item]['file'].ljust(54),uploaded_backups[item]['md5'].ljust(34),uploaded_backups[item]['sha1'].ljust(30)))
                # log("Cpanel user : %s, Eenos Backp : %s, md5sum : %s, sha1sum : %s" %(item,uploaded_backups[item]['file'],uploaded_backups[item]['md5'],uploaded_backups[item]['sha1']))
            